local _G = getfenv(0);

-- Bars
AzCastBar_Tokens = {};

-- Saved Variables
AzCastBar_Config = {};
local cfg;

-- Default Config
local AZCB_DefOptions = {
	enabled = true,
	showIcon = true,
	height = 20,
	fontSize = 12,
	alpha = 1,
	backdropIndent = -3,
	texture = "Interface\\Addons\\AzCastBar\\Textures\\Waterline",
	colBackdrop = { 0.1, 0.22, 0.35, 1.0 },
	colBackground = { 1.0, 1.0, 1.0, 1.0 },
	colProgress = { 0.4, 0.6, 0.8, 1.0 },
	colFont = { 1.0, 1.0, 1.0, 1.0 },
	colFailed = { 1.0, 0.5, 0.5, 1.0 },
	colInterrupt = { 1.0, 0.75, 0.5, 1.0 },
};

--------------------------------------------------------------------------------------------------------
--                                          OnLoad & OnEvent                                          --
--------------------------------------------------------------------------------------------------------
function AzCastBar_OnLoad()
	-- Init Specific Mod Vars
	this.modName = "AzCastBar";
	this.modVers = GetAddOnMetadata(this.modName,"Version");
	-- Events
	this:RegisterEvent("VARIABLES_LOADED");
	-- Add slash command
	SlashCmdList["AZCASTBAR"] = AzCastBar_OnSlash;
	SLASH_AZCASTBAR1 = "/azcb";
	SLASH_AZCASTBAR2 = "/acb";
end

-- Modify All CastBars to fit the Users Settings on "VARIABLES_LOADED"
function AzCastBar_OnEvent(event)
	AzCastBar_CheckSettings(AzCastBar_Config);
	AzCastBar_ApplyAllSettings();
end

--------------------------------------------------------------------------------------------------------
--                                           Slash Handling                                           --
--------------------------------------------------------------------------------------------------------
function AzCastBar_OnSlash(cmd)
	-- Extract Parameters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Options
	if (param1 == "") then
		local loaded, reason = LoadAddOn("AzCastBarOptions");
		if (loaded) then
			if (AzCastBarOptions:IsVisible()) then
				AzCastBarOptions:Hide();
			else
				AzCastBarOptions:Show();
			end
		else
			AzMsg("Could not option AzCastBar Options: |1"..tostring(reason));
		end
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg("----- |2"..AzCastBar.modName.."|r |1"..AzCastBar.modVers.."|r ----- |1"..format("%.2f",GetAddOnMemoryUsage(AzCastBar.modName)).." |2kb|r -----");
		AzMsg("The following |2parameters|r are valid for |2/azcb|r...");
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Apply Settings                                           --
--------------------------------------------------------------------------------------------------------

-- Checks if the settings exists and are correct
function AzCastBar_CheckSettings(cfgTable)
	-- Set new config if one given
	if (cfgTable) then
		cfg = cfgTable;
	end
	-- Check each frame has settings, reset to default if they do not
	local cfgEntry;
	for _, token in ipairs(AzCastBar_Tokens) do
		if (not cfg[token]) then
			cfg[token] = {};
		end
		cfgEntry = cfg[token];
		-- Options with different defaults
		if (cfgEntry.width == nil) then
			cfgEntry.width = (token == "Mirror" and 260 or 320);
		end
		if (cfgEntry.left == nil) then
			cfgEntry.left = _G["AzCastBar"..(token == "Mirror" and token.."1" or token)]:GetLeft();
		end
		if (cfgEntry.bottom == nil) then
			cfgEntry.bottom = _G["AzCastBar"..(token == "Mirror" and token.."1" or token)]:GetBottom();
		end
		if (cfgEntry.fadeTime == nil) then
			cfgEntry.fadeTime = (token == "Mirror" and 1.2 or 0.6);
		end
		if (token == "Player") and (cfgEntry.safeZone == nil) then
			cfgEntry.safeZone = true;
		end
		if (token ~= "Mirror") and (cfgEntry.showRank == nil) then
			cfgEntry.showRank = true;
		end
		-- Options with constant defaults
		for optionName, defOption in pairs(AZCB_DefOptions) do
			if (cfgEntry[optionName] == nil) then
				cfgEntry[optionName] = defOption;
			end
		end
	end
end

-- Apply Settings to All Bars
function AzCastBar_ApplyAllSettings()
	for _, token in ipairs(AzCastBar_Tokens) do
		AzCastBar_ApplyBarSettings(token);
	end
end

-- Apply Settings to Given Bar
function AzCastBar_ApplyBarSettings(frameToken)
	if (frameToken == "Mirror") then
		for i = 1, 3 do
			AzCastBar_ApplyBarSettingsSpecific(frameToken,i);
		end
	else
		AzCastBar_ApplyBarSettingsSpecific(frameToken);
	end
end

-- Apply Settings to Given Bar
function AzCastBar_ApplyBarSettingsSpecific(barToken,id)
	local cfgEntry = cfg[barToken];
	if (id) then
		barToken = barToken..id;
	end
	-- Give the Frame a Pointer to it's Config
	_G["AzCastBar"..barToken].cfg = cfgEntry;
	-- Width + Height
	_G["AzCastBar"..barToken]:SetWidth(cfgEntry.width);
	_G["AzCastBar"..barToken]:SetHeight(cfgEntry.height);
	-- Position
	if (not id or id == 1) then
		_G["AzCastBar"..barToken]:ClearAllPoints();
		_G["AzCastBar"..barToken]:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",cfgEntry.left,cfgEntry.bottom);
		_G["AzCastBar"..barToken]:SetBackdropColor(unpack(cfgEntry.colBackdrop));
	end
	-- Icon
	AzCastBar_ShowIcon(barToken,cfgEntry.showIcon);
	_G["AzCastBar"..barToken.."Icon"]:SetWidth(cfgEntry.height);
	-- Font Size
	local fontName, _, fontFlags = _G["AzCastBar"..barToken.."StatusLeft"]:GetFont();
	_G["AzCastBar"..barToken.."StatusLeft"]:SetFont(fontName,cfgEntry.fontSize,fontFlags);
	_G["AzCastBar"..barToken.."StatusRight"]:SetFont(fontName,cfgEntry.fontSize,fontFlags);
	-- Alpha
	_G["AzCastBar"..barToken]:SetAlpha(cfgEntry.alpha);
	-- Outline
	local backdrop = _G["AzCastBar"..barToken]:GetBackdrop();
	backdrop.insets.left, backdrop.insets.right, backdrop.insets.top, backdrop.insets.bottom = cfgEntry.backdropIndent, cfgEntry.backdropIndent, cfgEntry.backdropIndent, cfgEntry.backdropIndent;
	_G["AzCastBar"..barToken]:SetBackdrop(backdrop);
	-- Texture
	_G["AzCastBar"..barToken.."StatusTexture"]:SetTexture(cfgEntry.texture);
	-- Colors
	_G["AzCastBar"..barToken]:SetBackdropColor(unpack(cfgEntry.colBackdrop));
	_G["AzCastBar"..barToken.."StatusBackground"]:SetVertexColor(unpack(cfgEntry.colBackground));
--	_G["AzCastBar"..barToken.."StatusSafezone"]:SetVertexColor(unpack(cfgEntry.colSafezone));
	_G["AzCastBar"..barToken.."Status"]:SetStatusBarColor(unpack(cfgEntry.colProgress));
	_G["AzCastBar"..barToken.."StatusRight"]:SetTextColor(unpack(cfgEntry.colFont));
	_G["AzCastBar"..barToken.."StatusLeft"]:SetTextColor(unpack(cfgEntry.colFont));
end

-- Set Icon Visiblity for the given frame
function AzCastBar_ShowIcon(frameToken,show)
	local icon = _G["AzCastBar"..frameToken.."Icon"];
	if (show) then
		_G["AzCastBar"..frameToken.."Status"]:SetPoint("TOPLEFT",icon,"TOPRIGHT",3,0);
		icon:Show();
	else
		_G["AzCastBar"..frameToken.."Status"]:SetPoint("TOPLEFT");
		icon:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                                Misc                                                --
--------------------------------------------------------------------------------------------------------

-- Format time as it is shown on the CastBar
function AzCastBar_FormatTime(sec)
	if (sec <= 60) then
		return ("%.1f"):format(sec);
	else
		return ("%d:%.2d"):format(sec/60,mod(sec,60));
	end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 2)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 2) then
	AZMSG_REV = 2;
	function AzMsg(text)
		text = tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff");
		DEFAULT_CHAT_FRAME:AddMessage(text,128/255,192/255,255/255);
	end
end