local _G = getfenv(0);

-- Config
AzCastBar_Profiles = {};
local cfg = AzCastBar_Config;

-- Config Tables
local AZCB_Checks = {
	{ var = "enabled", label = "Enable Bar", tip = "Enable or disable this bar" },
	{ var = "showIcon", label = "Show Spell Icon", tip = "This option will display the spell icon to the left on the cast bar." },
	{ var = "showRank", label = "Show Spell Rank", tip = "If the spell being cast has a rank, it will be shown in brackets after the spell name." },
	{ var = "safeZone", label = "Show Safe Zone Area", tip = "The 'Safe Zone' is the time equal to your latency, with this option enabled, it will show this duration on the cast bar. A spell canceled after it has reached the safe zone, will still go off." },
};

local AZCB_Sliders = {
	{ var = "width", label = "Width", min = 20, max = 2048 },
	{ var = "height", label = "Height", min = 6, max = 120 },
	{ var = "left", label = "Left Offset", min = 0, max = 2048 },
	{ var = "bottom", label = "Bottom Offset", min = 0, max = 1536 },
	{ var = "fontSize", label = "Font Size", min = 4, max = 29 },
	{ var = "fadeTime", label = "Fade Out Time", min = 0.1, max = 4, step = 0.1 },
	{ var = "alpha", label = "Alpha", min = 0, max = 1, step = 0.01 },
	{ var = "backdropIndent", label = "Backdrop Indent", min = -20, max = 60, step = 0.5 },
};

local AZCB_Colors = {
	{ var = "colBackdrop", label = "Backdrop" },
	{ var = "colBackground", label = "Background" },
	{ var = "colProgress", label = "Progress" },
	{ var = "colFont", label = "Font" },
	{ var = "colFailed", label = "Failed" },
	{ var = "colInterrupt", label = "Interrupted" },
	--{ var = "colSafezone", label = "Safe Zone" },
}

local AZCB_Textures = {
	"Interface\\TargetingFrame\\UI-StatusBar",
	"Interface\\Addons\\AzCastBar\\Textures\\Test",
	"Interface\\Addons\\AzCastBar\\Textures\\Bricks",
	"Interface\\Addons\\AzCastBar\\Textures\\HorizontalFade",
	"Interface\\Addons\\AzCastBar\\Textures\\Leafs",
	"Interface\\Addons\\AzCastBar\\Textures\\Pale",
	"Interface\\Addons\\AzCastBar\\Textures\\Lines",
	"Interface\\Addons\\AzCastBar\\Textures\\Sunk",
	"Interface\\Addons\\AzCastBar\\Textures\\Raised",
	"Interface\\Addons\\AzCastBar\\Textures\\Smooth",
	"Interface\\Addons\\AzCastBar\\Textures\\Streamline",
	"Interface\\Addons\\AzCastBar\\Textures\\Streamline-Inverted",
	"Interface\\Addons\\AzCastBar\\Textures\\Waterline",
};

-- Data Variables
local ActiveBar = "Player";
local ProfilesList;

--------------------------------------------------------------------------------------------------------
--                                            Frame Scripts                                           --
--------------------------------------------------------------------------------------------------------
function AzCastBarOptions_OnLoad()
	AzCastBarOptionsVers:SetText("|cffffff80"..AzCastBar.modVers);
	if (cfg["Options"] and cfg["Options"].left and cfg["Options"].bottom) then
		this:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",cfg["Options"].left,cfg["Options"].bottom);
	end
end

-- OnShow
function AzCastBarOptions_OnShow()
	-- Init DropDown
	UIDropDownMenu_Initialize(AzCastBarOptionsDropDown,AzCastBarOptions_DropDown_Initialize);
	UIDropDownMenu_SetWidth(160,AzCastBarOptionsDropDown);
	-- Load Settings
	AzCastBarOptions_LoadSettingForFrame();
end

function AzCastBarOptions_OnMouseUp()
	this:StopMovingOrSizing();
	cfg["Options"] = { left = this:GetLeft(), bottom = this:GetBottom() };
end

--------------------------------------------------------------------------------------------------------
--                                           Check Buttons                                            --
--------------------------------------------------------------------------------------------------------
function AzCastBarOptions_ConfigCheckBox_OnLoad()
	_G[this:GetName().."Text"]:SetText(AZCB_Checks[this:GetID()].label);
	this.tooltipText = AZCB_Checks[this:GetID()].tip;
end

function AzCastBarOptions_ConfigCheckBox_OnClick()
	cfg[ActiveBar][AZCB_Checks[this:GetID()].var] = (this:GetChecked() == 1);
	AzCastBar_ApplyBarSettings(ActiveBar);
end

--------------------------------------------------------------------------------------------------------
--                                            Slider Frame                                            --
--------------------------------------------------------------------------------------------------------

-- Slider Frame OnLoad
function AzCastBarOptions_Slider_OnLoad()
	local id = this:GetID();
	local slider = (this:GetName().."Slider");

	_G[slider]:SetMinMaxValues(AZCB_Sliders[id].min,AZCB_Sliders[id].max);
	_G[slider]:SetValueStep(AZCB_Sliders[id].step or 1);

	_G[slider.."Text"]:SetText(AZCB_Sliders[id].label);
	_G[slider.."Low"]:SetText(AZCB_Sliders[id].min);
	_G[slider.."High"]:SetText(AZCB_Sliders[id].max);
end

-- Slider Value Changed
function AzCastBarOptions_Slider_OnValueChanged()
	-- Tooltip
	if (GameTooltip:IsOwned(this)) then
		GameTooltip:SetText(format("%.2f",this:GetValue()));
	end
	-- Change Edit
	_G[this:GetParent():GetName().."Edit"]:SetNumber(this:GetValue());
	-- Save & Apply
	if (not AzCastBar.loadingBarSetting) then
		cfg[ActiveBar][AZCB_Sliders[this:GetParent():GetID()].var] = this:GetValue();
		AzCastBar_ApplyBarSettings(ActiveBar);
	end
end

-- EditBox Enter Pressed
function AzCastBarOptions_SliderEdit_OnEnterPressed()
	_G[this:GetParent():GetName().."Slider"]:SetValue(this:GetNumber());
end

--------------------------------------------------------------------------------------------------------
--                                             Drop Down                                              --
--------------------------------------------------------------------------------------------------------
function AzCastBarOptions_DropDown_OnClick()
	UIDropDownMenu_SetSelectedValue(AzCastBarOptionsDropDown,this.value);
	cfg[ActiveBar].texture = this.value;
	AzCastBar_ApplyBarSettings(ActiveBar);
end

local info = { func = AzCastBarOptions_DropDown_OnClick };
function AzCastBarOptions_DropDown_Initialize()
	for _, texture in ipairs(AZCB_Textures) do
		info.text = texture:match("\\([^\\]+)$");
		info.value = texture;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
end
--------------------------------------------------------------------------------------------------------
--                                            Color Picker                                            --
--------------------------------------------------------------------------------------------------------

function AzCastBarOptions_Color_OnLoad()
	_G[this:GetName().."Border"]:SetVertexColor(0.6,0.6,0.6);
	_G[this:GetName().."Text"]:SetText(AZCB_Colors[this:GetID()].label);
end

function AzCastBarOptions_PickColor()
	local r, g, b, a = _G[this:GetName().."NormalTexture"]:GetVertexColor();

	ColorPickerFrame.frame = this;
	ColorPickerFrame.func = AzCastBarOptions_ColorPicker_Func;
	ColorPickerFrame.cancelFunc = AzCastBarOptions_ColorPicker_Func;
	ColorPickerFrame.opacityFunc = AzCastBarOptions_ColorPicker_Func;
	ColorPickerFrame.hasOpacity = true;
	ColorPickerFrame.opacity = (1 - a);
	ColorPickerFrame.previousValues = { r, g, b, a };

	OpacitySliderFrame:SetValue(1 - a);
	ColorPickerFrame:SetColorRGB(r,g,b);
	ColorPickerFrame:Show();
end

function AzCastBarOptions_ColorPicker_Func(prevVal)
	local r, g, b, a;
	if (prevVal) then
		r, g, b, a  = unpack(prevVal);
	else
		r, g, b = ColorPickerFrame:GetColorRGB();
		a = (1 - OpacitySliderFrame:GetValue());
	end
	_G[ColorPickerFrame.frame:GetName().."NormalTexture"]:SetVertexColor(r,g,b,a);
	cfg[ActiveBar][AZCB_Colors[ColorPickerFrame.frame:GetID()].var] = { r, g, b, a };
	AzCastBar_ApplyBarSettings(ActiveBar);
end

--------------------------------------------------------------------------------------------------------
--                                              Profiles                                              --
--------------------------------------------------------------------------------------------------------

-- Build the table used to display the profiles (display table)
function AzCastBarProfiles_BuildProfileList()
	-- Create display table
	ProfilesList = {};
	for entryName in pairs(AzCastBar_Profiles) do
		tinsert(ProfilesList,entryName);
	end
	sort(ProfilesList);
	-- Update
	AzCastBarProfilesHeader:SetText("Profiles ("..#ProfilesList..")");
	AzCastBarProfiles_ProfileList_Update();
	AzCastBarProfiles_OnTextChanged();
end

-- ScrollBar: Cache list update
function AzCastBarProfiles_ProfileList_Update()
	FauxScrollFrame_Update(AzCastBarProfilesScroll,#ProfilesList,8,18);
	local index, button;
	for i = 1, 8 do
		index = (FauxScrollFrame_GetOffset(AzCastBarProfilesScroll) + i);
		button = _G["AzCastBarProfilesEntry"..i];
		if (ProfilesList[index]) then
			_G["AzCastBarProfilesEntry"..i.."Text"]:SetText(ProfilesList[index]);
			button:SetWidth(#ProfilesList > 8 and 146 or 160);
			button:Show();
		else
			button:Hide();
		end
	end
end

-- Load
function AzCastBarProfiles_Load()
	local name = AzCastBarProfilesEdit:GetText();
	AzCastBar_Config = AzCastBarOptions_CopyTable(AzCastBar_Profiles[name]);
	cfg = AzCastBar_Config;
	AzCastBar_CheckSettings(AzCastBar_Config);
	AzCastBar_ApplyAllSettings();
	AzCastBarOptions_LoadSettingForFrame();
end

-- Save
function AzCastBarProfiles_Save()
	local name = AzCastBarProfilesEdit:GetText();
	AzCastBar_Profiles[name] = AzCastBarOptions_CopyTable(cfg);
	AzCastBarProfiles_BuildProfileList();
end

-- Delete
function AzCastBarProfiles_Delete()
	local name = AzCastBarProfilesEdit:GetText();
	AzCastBar_Profiles[name] = nil;
	AzCastBarProfiles_BuildProfileList();
end

-- Text Changed
function AzCastBarProfiles_OnTextChanged()
	local name = AzCastBarProfilesEdit:GetText();
	-- save
	if (name == "") then
		AzCastBarProfilesSave:Disable();
	else
		AzCastBarProfilesSave:Enable();
	end
	-- load & delete
	if (AzCastBar_Profiles[name]) then
		AzCastBarProfilesLoad:Enable();
		AzCastBarProfilesDelete:Enable();
	else
		AzCastBarProfilesLoad:Disable();
		AzCastBarProfilesDelete:Disable();
	end
end

--------------------------------------------------------------------------------------------------------
--                                              Edit Mode                                             --
--------------------------------------------------------------------------------------------------------

-- Toggle EditMode
function AzCastBarOptions_EditMode()
	local frame;
	for _, token in ipairs(AzCastBar_Tokens) do
		frame = "AzCastBar"..(token == "Mirror" and token.."1" or token);
		if (this:GetChecked()) then
			_G[frame]:EnableMouse(1);
			_G[frame]:SetMovable(1);

			_G[frame].oldOnUpdate = _G[frame]:GetScript("OnUpdate");
			_G[frame].oldOnEvent = _G[frame]:GetScript("OnEvent");
			_G[frame]:SetScript("OnUpdate",AzCastBarOptions_EditMode_OnUpdate);
			_G[frame]:SetScript("OnEvent",nil);

			_G[frame.."StatusLeft"]:SetText(token.." Bar");
			_G[frame.."Status"]:SetMinMaxValues(0,2);
			_G[frame.."Status"]:SetStatusBarColor(unpack(_G[frame].cfg.colProgress));
			_G[frame.."Icon"]:SetTexture("Interface\\Icons\\Spell_Nature_UnrelentingStorm");

			_G[frame]:SetAlpha(_G[frame].cfg.alpha);
			_G[frame]:Show();
		else
			_G[frame]:EnableMouse(0);
			_G[frame]:SetMovable(0);

			_G[frame]:SetScript("OnUpdate",_G[frame].oldOnUpdate);
			_G[frame]:SetScript("OnEvent",_G[frame].oldOnEvent);

			_G[frame]:Hide();
			_G[frame]:oldOnEvent("PLAYER_ENTERING_WORLD","");
			_G[frame].oldOnUpdate = nil;
			_G[frame].oldOnEvent = nil;
		end
	end
end

-- OnUpdate
function AzCastBarOptions_EditMode_OnUpdate(self,elapsed)
	_G[self:GetName().."Status"]:SetValue(mod(GetTime(),2));
	_G[self:GetName().."StatusRight"]:SetText(AzCastBar_FormatTime(mod(GetTime(),2)));
end

-- Stop Moving a Bar
function AzCastBarOptions_EditMode_OnMouseUp(self)
	self:StopMovingOrSizing();
	-- Save New Position
	cfg[self.token].left = self:GetLeft();
	cfg[self.token].bottom = self:GetBottom();
	-- Reflect Change in Options if Visible
	if (AzCastBarOptions:IsVisible()) and (self.token == ActiveBar) then
		AzCastBarOptionsSlider3Slider:SetValue(self:GetLeft());
		AzCastBarOptionsSlider4Slider:SetValue(self:GetBottom());
	end
end

--------------------------------------------------------------------------------------------------------
--                                                Misc                                                --
--------------------------------------------------------------------------------------------------------

-- Load Settings for the current frame token
function AzCastBarOptions_LoadSettingForFrame()
	_G["AzCastBarOptions"..ActiveBar.."Button"]:SetTextColor(0,1,0);
	-- Start Loading + Get cfgEntry
	AzCastBar.loadingBarSetting = 1;
	local cfgEntry = cfg[ActiveBar];
	-- Checks
	for index, table in ipairs(AZCB_Checks) do
		if (cfgEntry[table.var] == nil) then
			_G["AzCastBarOptionsCheck"..index]:Hide();
		else
			_G["AzCastBarOptionsCheck"..index]:SetChecked(cfgEntry[table.var]);
			_G["AzCastBarOptionsCheck"..index]:Show();
		end
	end
	-- Sliders
	for index, table in ipairs(AZCB_Sliders) do
		_G["AzCastBarOptionsSlider"..index.."Slider"]:SetValue(cfgEntry[table.var]);
	end
	-- Texture
	UIDropDownMenu_SetSelectedValue(AzCastBarOptionsDropDown,cfgEntry.texture);
	-- Colors
	for index, table in ipairs(AZCB_Colors) do
		_G["AzCastBarOptionsColor"..index.."NormalTexture"]:SetVertexColor(unpack(cfgEntry[AZCB_Colors[index].var]));
	end
	-- Done Loading
	AzCastBar.loadingBarSetting = nil;
end

-- Copy Table Function
function AzCastBarOptions_CopyTable(source)
	local dest = {};
	for k, v in pairs(source) do
		if (type(v) == "table") then
			dest[k] = AzCastBarOptions_CopyTable(v);
		else
			dest[k] = v;
		end
	end
	return dest;
end

-- "Select Frame" Buttons
function AzCastBarOptions_SelectButtons_OnClick()
	_G["AzCastBarOptions"..ActiveBar.."Button"]:SetTextColor(1,0.82,0);
	ActiveBar = AzCastBar_Tokens[this:GetID()];
	AzCastBarOptions_LoadSettingForFrame();
end

-- Resets the options for the current bar
function AzCastBarOptions_ResetBar()
	cfg[ActiveBar] = nil;
	AzCastBar_CheckSettings();
	AzCastBar_ApplyBarSettings(ActiveBar);
	AzCastBarOptions_LoadSettingForFrame();
end

-- Show/Hide Profiles
function AzCastBarOptions_Profiles()
	if (AzCastBarProfiles:IsVisible()) then
		AzCastBarProfiles:Hide();
	else
		AzCastBarProfiles:Show();
	end
end